@@ -30,9 +30,6 @@ so, delete this exception statement from your version.  */
 #ifndef FTP_H
 #define FTP_H
 
-/* Need it for struct rbuf.  */
-#include "rbuf.h"
-
 #include "host.h"
 
 /* System types. */
@@ -46,22 +43,28 @@ enum stype
   ST_OTHER
 };
   
-uerr_t ftp_response PARAMS ((struct rbuf *, char **));
-uerr_t ftp_login PARAMS ((struct rbuf *, const char *, const char *));
-uerr_t ftp_port PARAMS ((struct rbuf *));
-uerr_t ftp_pasv PARAMS ((struct rbuf *, ip_address *, unsigned short *));
+uerr_t ftp_response PARAMS ((int, char **));
+uerr_t ftp_login PARAMS ((int, const char *, const char *));
+uerr_t ftp_port PARAMS ((int, int *));
+uerr_t ftp_pasv PARAMS ((int, ip_address *, int *));
 #ifdef ENABLE_IPV6
-uerr_t ftp_epsv PARAMS ((struct rbuf *, ip_address *, unsigned short *,
-			 char *));
+uerr_t ftp_lprt PARAMS ((int, int *));
+uerr_t ftp_lpsv PARAMS ((int, ip_address *, int *));
+uerr_t ftp_eprt PARAMS ((int, int *));
+uerr_t ftp_epsv PARAMS ((int, ip_address *, int *));
+#endif
+uerr_t ftp_type PARAMS ((int, int));
+uerr_t ftp_cwd PARAMS ((int, const char *));
+uerr_t ftp_retr PARAMS ((int, const char *));
+uerr_t ftp_rest PARAMS ((int, wgint));
+uerr_t ftp_list PARAMS ((int, const char *));
+uerr_t ftp_syst PARAMS ((int, enum stype *));
+uerr_t ftp_pwd PARAMS ((int, char **));
+uerr_t ftp_size PARAMS ((int, const char *, wgint *));
+
+#ifdef ENABLE_OPIE
+const char *skey_response PARAMS ((int, const char *, const char *));
 #endif
-uerr_t ftp_type PARAMS ((struct rbuf *, int));
-uerr_t ftp_cwd PARAMS ((struct rbuf *, const char *));
-uerr_t ftp_retr PARAMS ((struct rbuf *, const char *));
-uerr_t ftp_rest PARAMS ((struct rbuf *, long));
-uerr_t ftp_list PARAMS ((struct rbuf *, const char *));
-uerr_t ftp_syst PARAMS ((struct rbuf *, enum stype *));
-uerr_t ftp_pwd PARAMS ((struct rbuf *, char **));
-uerr_t ftp_size PARAMS ((struct rbuf *, const char *, long int *));
 
 struct url;
 
@@ -78,7 +81,7 @@ enum ftype
 /* Globbing (used by ftp_retrieve_glob).  */
 enum
 {
-  GLOBALL, GETALL, GETONE
+  GLOB_GLOBALL, GLOB_GETALL, GLOB_GETONE
 };
 
 /* Information about one filename in a linked list.  */
@@ -86,7 +89,7 @@ struct fileinfo
 {
   enum ftype type;		/* file type */
   char *name;			/* file name */
-  long size;			/* file size */
+  wgint size;			/* file size */
   long tstamp;			/* time-stamp */
   int perms;			/* file permissions */
   char *linkto;			/* link to which file points */
@@ -101,9 +104,7 @@ enum wget_ftp_command
   DO_CWD        = 0x0002,	/* Change current directory.  */
   DO_RETR       = 0x0004,	/* Retrieve the file.  */
   DO_LIST       = 0x0008,	/* Retrieve the directory list.  */
-  LEAVE_PENDING = 0x0010,	/* Do not close the socket.  */
-  NO_TRUNCATE	= 0x0020	/* Don't truncate the file if REST
-				   malfunctions. */
+  LEAVE_PENDING = 0x0010	/* Do not close the socket.  */
 };
 
 enum wget_ftp_fstatus
